#include "socket_send.h"

#include <errno.h>
#include <sys/socket.h>
#include <time.h>

int socket_send(int socket, const uint8_t* data, size_t dlen)
{
    struct timespec tim = { 0, 50000000 }; /* 50 ms */
    ssize_t nbytes, todo = dlen;
    int ret, retries = 0;

    while (retries < 3 && todo > 0) {
        nbytes = send(socket, data, todo, MSG_NOSIGNAL);
        if (nbytes == -1) {
            ret = errno;
            if (ret == EAGAIN || ret == EWOULDBLOCK || ret == EINTR) {
                retries++;
                nanosleep(&tim , NULL);
                continue;
            }
            return -ret; /* peer has disconnected */
        }
        data += nbytes;
        todo -= nbytes;
    }

    if (todo > 0) {
        return -ETIMEDOUT;
    }
    return 0;
}

int socket_writev(int socket, const struct iovec *iov, int iovcnt)
{
    int i, err;

    for (i = 0; i < iovcnt; i++) {
        /* TODO writev is hard to use with retries */
        err = socket_send(socket, iov[i].iov_base, iov[i].iov_len);
        if (err) {
            return err;
        }
    }
    return 0;
}
